"use strict";

var GroupedStrategy = require("./ui.scheduler.work_space.grouped.strategy");

var VerticalGroupedStrategy = GroupedStrategy.inherit({
    prepareCellIndexes: function prepareCellIndexes(cellCoordinates, groupIndex) {
        var rowIndex = cellCoordinates.rowIndex + groupIndex * this._workSpace._getRowCount();

        if (this._workSpace.supportAllDayRow() && this._workSpace.option("showAllDayPanel")) {
            rowIndex += groupIndex;
        }

        return {
            rowIndex: rowIndex,
            cellIndex: cellCoordinates.cellIndex
        };
    },

    calculateCellIndex: function calculateCellIndex(rowIndex, cellIndex) {
        rowIndex = rowIndex % this._workSpace._getRowCount();

        return this._workSpace._getRowCount() * cellIndex + rowIndex;
    },

    getGroupIndex: function getGroupIndex(rowIndex, cellIndex) {
        return Math.floor(rowIndex / this._workSpace._getRowCount());
    },

    calculateHeaderCellRepeatCount: function calculateHeaderCellRepeatCount() {
        return 1;
    },

    insertAllDayRowsIntoDateTable: function insertAllDayRowsIntoDateTable() {
        return this._workSpace.option("showAllDayPanel");
    },

    getTotalCellCount: function getTotalCellCount(groupCount) {
        return this._workSpace._getCellCount();
    },

    getTotalRowCount: function getTotalRowCount() {
        return this._workSpace._getRowCount() * this._workSpace._getGroupCount();
    },

    addAdditionalGroupCellClasses: function addAdditionalGroupCellClasses(cellClass, index, i, j) {
        cellClass = this._addLastGroupCellClass(cellClass, i + 1);

        return this._addFirstGroupCellClass(cellClass, i + 1);
    },

    _addLastGroupCellClass: function _addLastGroupCellClass(cellClass, index) {
        if (index % this._workSpace._getRowCount() === 0) {
            return cellClass + " " + this.getLastGroupCellClass();
        }

        return cellClass;
    },

    _addFirstGroupCellClass: function _addFirstGroupCellClass(cellClass, index) {
        if ((index - 1) % this._workSpace._getRowCount() === 0) {
            return cellClass + " " + this.getFirstGroupCellClass();
        }

        return cellClass;
    },

    getHorizontalMax: function getHorizontalMax(groupIndex) {
        return this._workSpace.getMaxAllowedPosition()[0];
    },

    getVerticalMax: function getVerticalMax(groupIndex) {
        return this._workSpace.getMaxAllowedVerticalPosition()[groupIndex];
    },

    calculateTimeCellRepeatCount: function calculateTimeCellRepeatCount() {
        return this._workSpace._getGroupCount() || 1;
    }
});

module.exports = VerticalGroupedStrategy;